/* stfpio.h - bc635PCI Solaris 8 - 10 device driver header file */

/* Version 1.0: 12 May 94     */
/* Version 1.1: 18 Jan 95     */
/* Version 1.2: 05 May 98     */
/* Version 3.1: 15 March 2000 */
/* Version 4.4: 06 November 2000 */
/* Version 7.0: 27 Mar 2008   */
/* Version 8.0: 27 Mar 2009   */

/*
#include <sys/time.h>
*/

/*-------------------------------------------------------------*/
/* data structures */

struct	stfp_tm {
    int32_t tm_sec; 
    int32_t tm_min;
    int32_t tm_hour;
    int32_t tm_mday;
    int32_t tm_mon;
    int32_t tm_year;
    int32_t tm_wday;
    int32_t tm_yday;
    int32_t tm_isdst;
};

struct	stfp_dec_tm {
    uint16_t dec_year;
    uint16_t dec_day;
    u_char   dec_hour;
    u_char   dec_min;
    u_char   dec_sec;
};

struct stfp_time {
    struct stfp_tm tm;
    int32_t        usec;        /* usec 0 - 999999 */
    int32_t        hnsec;       /* hnsec 0 - 9 (hundreds of nsecs) */
    int32_t        status;
};

struct stfp_tmval {	
    int32_t tv_sec;
    int32_t tv_usec;
};	

struct stfp_timeval {
    struct stfp_tmval tv; 
    int32_t           hnsec;
    int32_t           status;
};

struct periodic {
    u_char   sync1pps;          /* TRUE = sync periodic to 1pps */
    uint16_t n1;                /* divider n1 */
    uint16_t n2;                /* divider n2 */
};

/* Additional Structures to interpret GETDATA command results */
/*	GETDATA_UTCINFO	0x18 */
struct gd_utcinfo {
    u_char   enable;
    char     leapsec;
    char     leapevt;
    uint32_t evttime;
};

/* GETDATA_TCGENOFF  0x1C */
struct tcgenoffset {
    uint16_t genoff;            /* local offset -16 to +16 Hours  */
    u_char   genflg;            /* flag: 1: half_hour, 0: no_half_hour */
};

/* GETDATA_LOCTMOFF  0x1D */
struct loctmoffset {
    uint16_t locoff;            /* Generator offset -16 to +16 Hours  */
    u_char   locflg;            /* flag: 1: half_hour, 0: no_half_hour */
};

/* GETDATA_LEAPSEC   0x1E */
struct leapseconds {
    char     ls_flag;           /* 1: Insertion, -1: Deletion, 0: Disable */
    uint32_t leap_tm;           /* Leap Event Time - unix format */
};

/*	GETDATA_FWVER	0x1F */
struct gd_fwversion {
    u_char   vmajor;            /* major version number */
    u_char   vminor;            /* minor version number */
    u_char   month;             /* month (1 - 12) */
    u_char   day;               /* day (1 - 31 ) */
    uint16_t year;              /* year (e.g. 1994) */
};

/*	GETDATA_OSCCTL	0x23 */
struct gd_oscdisctl {
    char     discctl;
    char     phasectl;
    uint16_t phasestep;
    int16_t  gain;
    int32_t  con;
};

/* GETDATA_DTFW	  0x4F */
struct dtfwrev {
    u_char dt01;                /* 'D' */
    u_char dt02;                /* 'T' */
    u_char dt03;                /* '6' */
    u_char dt04;                /* '0' */
    u_char dt05;                /* '0' */
    u_char dt06;                /* '0' */
    u_char dt07;                /* Rev */
    u_char dt08;                /* Rev */
    u_char dt09;                /* Rev */
    u_char dt10;                /* Rev */
    u_char dt11;                /* Rev */
};

/* GETDATA_TFPMODEL  0xF6 */
struct tfpmodel {
    u_char tfp1;                /* 'B' */
    u_char tfp2;                /* 'C' */
    u_char tfp3;                /* '6' */
    u_char tfp4;                /* '3' */
    u_char tfp5;                /* '5' or '7' */
    u_char tfp6;                /* 'P' */
    u_char tfp7;                /* 'C' */
    u_char tfp8;                /* 'I' */
};


/* Passed as arg in the GETDATA (38) command */
struct getdata_t {
    u_char arg; /* Get Data Command Argument */
    u_char padding[3];
    union
    {
        u_char tmode;                /* GETDATA_MODE      0x10 */
        u_char tformat;              /* GETDATA_TFORMAT   0x11 */
        uint16_t year;               /* GETDATA_YEAR      0x13 */
        struct periodic period;      /* GETDATA_PER       0x14 */
        u_char tcformat[2];          /* GETDATA_TCFMT     0x15 */
        char tcmod;                  /* GETDATA_TCMOD     0x16 */
        int32_t offset;              /* GETDATA_TOFF      0x17 */ 
        struct gd_utcinfo utc;       /* GETDATA_UTCINFO   0x18 */

        u_char tcoutfmt[2];          /* GETDATA_TCOUTFMT  0x1B */
        struct tcgenoffset genoff;   /* GETDATA_TCGENOFF  0x1C */
        struct loctmoffset locoff;   /* GETDATA_LOCTMOFF  0x1D */
        struct leapseconds leapsec;  /* GETDATA_LEAPSEC   0x1E */

        struct gd_fwversion version; /* GETDATA_FWVER     0x1F */
        char clksrc;                 /* GETDATA_CLKSRC    0x20 */
        char jsctl;                  /* GETDATA_JAMSC     0x21 */
        struct gd_oscdisctl ocsctl;  /* GETDATA_OSCCTL    0x23 */
        uint16_t davalue;            /* GETDATA_DAVAL     0x24 */

        u_char battstat;             /* GETDATA_BATTSTAT  0x26 */
        int32_t clkval;              /* GETDATA_CLKVAL    0x29 */
        struct dtfwrev dtfw;         /* GETDATA_DTFW	  0x4F */
        uint16_t assemb;             /* GETDATA_ASSEMB    0xF4 */
        uint16_t hwfab;	             /* GETDATA_HWFAB     0xF5 */
        struct tfpmodel tfpm;        /* GETDATA_TFPMODEL  0xF6 */
        uint32_t serialnum;          /* GETDATA_SERIAL    0xFE */
    } data;
};

struct stfp_version {
    u_char   vmajor;            /* major version number */
    u_char   vminor;            /* minor version number */
    u_char   month;             /* month (1 - 12) */
    u_char   day;               /* day (1 - 31 ) */
    uint16_t year;              /* year (e.g. 1994) */
};

#define TM_YEAR_BASE    1900
#define STFP_MAX_READ   80
#define STFP_MAX_WRITE  80

// Passed as arg in the RDDPOFFSET and WRDPOFFSET command
struct dp_rdwr_t {
    uint16_t offset;
    u_char   data;
};

// Passed as arg in the RDREGOFFSET and WRREGOFFSET command
struct reg_rdwr_t {
    uint16_t offset;
    uint32_t data;
};

// jsy - 04/02/2008
// In truth, we should create a single data structure that can be
// used in all dual port RAM command. The structure would have a byte
// for the command and an array of bytes for the data specific for the
// command. That way, we should have only one 'ioctl' code, something
// like 'DPRAMCOMMAND'. However, since there are applications already
// written that relies on the existing 'ioctl' code, any change would
// break those applications. Therefore, the following structures are
// to extend what we have to cover features in the new V2 hardware.

// Passed as arg in the DDSCOMMAND command
struct dds_command_t {
    u_char  cmd_id;
    uint8_t cmd_len;
    union {
        u_char   cmd_byte[4];
        uint32_t cmd_number;
    } data;
};

// Passed as arg in the TCFORMATEX command. This ioctl is used
// for selecting either input format or output format.
struct tc_command_t {
    u_char cmd_id;          // Input or output
    u_char type;            // Time code
    u_char subtype;         // IRIG code subtype or TCODE_IRIG_SUBTYPE_NONE
};

// Data structure for events control.
struct EventsData {
    u_char evtsrc;          // Event trigger source (0=evt,1=hbt)
    u_char evtctl;          // Enable/Disable events
    u_char evtsense;        // Event trigger on edge (1=falling,0=rising)
    u_char evtlock;         // Enable/Disable event capture lockout
    u_char evt2ctl;         // Enable/Disable event2
    u_char evt2sense;       // Event2 trigger on edge (1=falling,0=rising)
    u_char evt2lock;        // Enable/Disable event2 capture lockout
    u_char evt3ctl;         // Enable/Disable event3
    u_char evt3sense;       // Event3 trigger on edge (1=falling,0=rising)
    u_char evt3lock;        // Enable/Disable event3 capture lockout
};

/*-------------------------------------------------------------*/
/* ioctl() commands */

#define SELTIMINGMODE	1	
#	define MODE_TIMECODE	0
#	define MODE_FREERUN	1
#	define MODE_EXT1PPS	2
#	define MODE_RTC		3
#	define MODE_GPS		6

#define SELTIMEFORMAT	2	
#	define TIME_DECIMAL	0
#	define TIME_BINARY	1

#define TIMEREQUEST		3
#define EVENTREQUEST		4

#define RDTIME			5
#define RDEVENT			6

#define RDTIMETV		7
#define RDEVENTTV		8

#define RDTIMEREQ		9
#define RDEVENTREQ		10

#define RDTIMETVREQ		11
#define RDEVENTTVREQ		12

#define WRSTROBE		13
#define WRSTROBETV		14

#define SELTCFORMAT	15
#	define TC_IRIGA		'A'
#	define TC_IRIGB		'B'
#	define TC_IEEE		'I'
#	define TC_NASA		'N'

#define SELTCMOD	16
#	define MOD_AM		'M'
#	define MOD_DCLS		'D'

#define SETTIME			17	
#define SETTIMETV		18
#define SETYEAR			19
#define SETLOCALOFFSET	20

#define SETPERIODIC	21
#	define PERIODIC_NOSYNC	0
#	define PERIODIC_SYNC	1

#define SETTIMINGOFFSET	22

#define SELFREQUENCYOUT	23
#	define FREQ_10MHZ	1
#	define FREQ_5MHZ	2
#	define FREQ_1MHZ	3

#define CONTROLEVENT	25
#	define EVENT_RISING	1
#	define EVENT_FALLING	2
#	define CAPLOCK_ENABLE	3
#	define CAPLOCK_DISABLE	4
#	define EVCAP_ENABLE	5
#	define EVCAP_DISABLE	6
#	define EVCAP_PERIODIC	7
#	define EVCAP_EVENT	8

#define CONTROLSTROBE	26
#	define STROBE_SECUS	1
#	define STROBE_USONLY	2
#	define STROBE_ENABLE	3
#	define STROBE_DISABLE	4

#define CAPUNLOCK		27
#define SETINTSIGNAL		28
#define RDINTSIGNAL		29
#define RDINTSTAT		30
#define CLRINTSTAT		31
#define RDFIRMVERSION		32

#define CONTROLTIMEBASE	33
#	define CLOCK_INTERNAL		1
#	define CLOCK_EXTERNAL		2
#	define JAMSYNC_ENABLE		3
#	define JAMSYNC_DISABLE		4
#	define DISCIPLINE_ENABLE	5
#	define DISCIPLINE_DISABLE	6
#	define FORCE_JAMSYNC		7
#	define CLEAR_USEC		8

#define SETDAC			34
#define RDDAC			35
#define SETDISCGAIN		36
#define REQGPSPACKET		37

#define GETDATA	38
#	define 	GETDATA_MODE		0x10
#	define 	GETDATA_TFORMAT		0x11
#	define 	GETDATA_YEAR		0x13
#	define 	GETDATA_PER		0x14
#	define 	GETDATA_TCFMT		0x15
#	define 	GETDATA_TCMOD		0x16
#	define 	GETDATA_TOFF		0x17
#	define 	GETDATA_UTCINFO		0x18

#	define	GETDATA_TCOUTFMT	0x1B
#	define	GETDATA_TCGENOFF	0x1C
#	define	GETDATA_LOCTMOFF	0x1D
#	define	GETDATA_LEAPSEC		0x1E

#	define 	GETDATA_FWVER		0x1F
#	define 	GETDATA_CLKSRC		0x20
#	define 	GETDATA_JAMSC		0x21
#	define 	GETDATA_OSCCTL		0x23
#	define 	GETDATA_DAVAL		0x24

#	define	GETDATA_BATTSTAT	0x26
#	define	GETDATA_CLKVAL		0x29
#	define	GETDATA_DTFW		0x4F
#	define	GETDATA_ASSEMB		0xF4
#	define	GETDATA_HWFAB		0xF5 
#	define	GETDATA_TFPMODEL	0xF6
#	define	GETDATA_SERIAL		0xFE

/*-------------------------------------------------------------*/

#define	SOFTWARERESET	39
#define SW_RESET	1	

#define	SETTCOUTFMT	40
#	define	TC_OUT_IRIGB	0x42
#	define	TC_OUT_IEEE	0x49

#define	SETGENTMOFFSET	41
#define	SETLOCTMOFFSET	42
#define	SETLEAPSECEVENT	43
#define	SETDISGAIN		44	
#define	SETCLKVAL		45	

#define	SETGPSTMFMT	46	
#	define	UTC_FMT		0
#	define	GPS_FMT		1

#define	SETGPSMDFLG	47	
#	define	GPS_FLG_DIS	0
#	define	GPS_FLG_ENA	1

#define	SETLOCTMFLG	48
#	define	LOC_OFF_DIS	0
#	define	LOC_OFF_ENA	1

#define	SETYRINCFLG	49
#	define	YR_INC_DIS	0
#	define	YR_INC_ENA	1		

#define	SYNCRTC			50
#define	DISCBATT		51	
#define	REQBATTSTS		52
#define	REQLOCTMFLG		53	
#define	REQFWNUM		54	
#define	REQASSEMNUM		55
#define	REQHWFAB		56
#define	REQTFPMODEL		57
#define	REQTFPSERIAL	        58
#define	SETDECTIME		59
#define RDCONTROL		60
#define SETCONTROL		61
#define REQREVID		62
#define RDDPOFFSET		63
#define WRDPOFFSET		64
#define SENDCOMMAND		65
#define DDSCOMMAND		66

//-------------------------------------------------------------
// The TCFORMATEX supports selection of time code input and/or
// output that are available in the V2 hardware. The V2 hardware
// supports everything supported by the V1 hardware and much more.
// Some time code (input or output) can have a sub type.
//
// For time code input, in addition to V1 suppored IRIG A, IRIG B,
// IEEE 1344 and NASA 36, V2 hardware supports 
//   IRIG AY (IRIG A with year)
//   IRIG BY (IRIG B with year)
//   IRIG BT (IRIG B Legacy TrueTime)
//   IRIG E  (IRIG E 1000Hz no year)
//   IRIG EY (IRIG E 1000Hz with year)
//   IRIG e  (IRIG E 100Hz no year)
//   IRIG eY (IRIG E 100Hz with year)
//   IRIG G  (IRIG G no year)
//   IRIG GY (IRIG G with year)
//   XR3
//   2137
//
// For time code output, in addition to V1 supported IRIG B and
// IEEE 1344, V2 hardware supports
//   IRIG A0 (IRIG A - BCD,CF,SBS)
//   IRIG A1 (IRIG A - BCD,CF)
//   IRIG A2 (IRIG A - BCD)
//   IRIG A3 (IRIG A - BCD,SBS)
//   IRIG A4 (IRIG A - BCD,YEAR,CF,SBS)
//   IRIG A5 (IRIG A - BCD,YEAR,CF)
//   IRIG A6 (IRIG A - BCD,YEAR)
//   IRIG A7 (IRIG A - BCD,YEAR,SBS)
//   IRIG B0 (IRIG B - BCD,CF,SBS)
//   IRIG B1 (IRIG B - BCD,CF)
//   IRIG B2 (IRIG B - BCD)
//   IRIG B3 (IRIG B - BCD,SBS)
//   IRIG B4 (IRIG B - BCD,YEAR,CF,SBS)
//   IRIG B5 (IRIG B - BCD,YEAR,CF)
//   IRIG B6 (IRIG B - BCD,YEAR)
//   IRIG B7 (IRIG B - BCD,YEAR,SBS)
//   IRIG BT (IRIG B - BCD,CF,SBS Legacy TrueTime)
//   IRIG E1 (IRIG E - 1000Hz BCD,CF)
//   IRIG E2 (IRIG E - 1000Hz BCD)
//   IRIG E5 (IRIG E - 1000Hz BCD,YEAR,CF)
//   IRIG E6 (IRIG E - 1000Hz BCD,YEAR)
//   IRIG e1 (IRIG E - 100Hz BCD,CF)
//   IRIG e2 (IRIG E - 100Hz BCD)
//   IRIG e5 (IRIG E - 100Hz BCD,YEAR,CF)
//   IRIG e6 (IRIG E - 100Hz BCD,YEAR)
//   IRIG G5 (IRIG G - BCD,YEAR,CF)
//   NASA 36
//   XR3
//   2137
#define TCFORMATEX		67
// ADDITIONAL TIME CODE TYPE FOR V2 HARDWARE
#	define TC_2137		'2'
#	define TC_IRIGE		'E'
#	define TC_IRIGG		'G'
#	define TC_XR3		'X'
#	define TC_IRIGe		'e'
// Time Code Sub Type
#	define TC_SUBTYPE_NONE	0
#	define TC_SUBTYPE_Y	'Y'
#	define TC_SUBTYPE_T	'T'
#	define TC_SUBTYPE_0	'0'
#	define TC_SUBTYPE_1	'1'
#	define TC_SUBTYPE_2	'2'
#	define TC_SUBTYPE_3	'3'
#	define TC_SUBTYPE_4	'4'
#	define TC_SUBTYPE_5	'5'
#	define TC_SUBTYPE_6	'6'
#	define TC_SUBTYPE_7	'7'

#define RDREGOFFSET		68
#define WRREGOFFSET		69
#define RDEVENT2TV		70
#define RDEVENT3TV		71

/*-------------------------------------------------------------*/
/* bc635PCI DPRAM Commands */

#define COM_SETTIMINGMODE	0x10
#define COM_SETTIMEFORMAT	0x11
#define COM_SETTIME		0x12
#define COM_SETYEAR		0x13
#define COM_SETPERIODIC		0x14
#define COM_SETTCFORMAT		0x15
#define COM_SETTCMOD		0x16
#define COM_SETTIMINGOFFSET	0x17
#define COM_REQUTCTMDATA	0x18	
#define COM_GETDATA		0x19
#	define GETDATA_DAC	0
#	define GETDATA_VERSION	2

#define	COM_SOFTRESET		0x1A  
#define COM_SETTCOUTFMT		0x1B
#define COM_SETGENTMOFF		0x1C
#define	COM_SETLOCTMOFF		0x1D
#define	COM_SETLEAPSEC		0x1E

#define COM_SELCLOCK		0x20
#	define SELCLOCK_INT	'I'
#	define SELCLOCK_EXT	'E'
#define COM_JAMSYNCENABLE	0x21
#	define JAMSYNC_DIS	0
#	define JAMSYNC_EN	1
#define COM_FORCEJAMSYNC	0x22
#define	COM_DISCENABLE		0x23
#	define DISC_DIS		0
#	define DISC_EN		1
#define COM_SETDAC		0x24
#define COM_SETDISCGAIN		0x25

#define	COM_REQBATTSTAT		0x26
#define COM_SYNCRTC		0x27
#define COM_DISCBATTRTC		0x28
#define COM_SETCLKVAL		0x29

#define COM_SENDGPSPACKET	0x30
#define COM_REQGPSPACKET	0x31
#define COM_MANREQGPSPACKET	0x32
#define COM_GPSTIMEFORMAT	0x33

#define COM_SETGPSMDFLG		0x34
#define COM_SETLOCTMFLG		0x40
#define	COM_REQDAYLTFLG		0x41
#define COM_SETYRINCFLG		0x42
#define COM_SET_PRD_DDS_SEL     0x43
#       define SELECT_PERIODIC_OUT      0x0
#       define SELECT_DDS_OUT           0x1
#define COM_SET_PRD_DDS_ENA     0x44
#define COM_SET_DDS_DIVIDER     0x45
#       define DDS_DIVIDE_BY_1E0        0x0
#       define DDS_DIVIDE_BY_1E1        0x1
#       define DDS_DIVIDE_BY_1E2        0x2
#       define DDS_DIVIDE_BY_1E3        0x3
#       define DDS_DIVIDE_BY_1E4        0x4
#       define DDS_DIVIDE_BY_1E5        0x5
#       define DDS_DIVIDE_BY_1E6        0x6
#       define DDS_DIVIDE_BY_1E7        0x7
#       define DDS_DIVIDE_BY_PREG       0xF
#define COM_SET_DDS_DIV_SRC     0x46
#       define DDS_DIVIDER_SRC_DDS      0x0
#       define DDS_DIVIDER_SRC_MULT     0x1
#       define DDS_DIVIDER_SRC_100MHZ   0x2
#define COM_SET_DDS_DIV_SYNC    0x47
#       define DDS_SYNC_MODE_FRAC       0x0
#       define DDS_SYNC_MODE_CONT       0x1
#define COM_SET_DDS_MULTI       0x48
#       define DDS_MULTIPLY_BY_1        0x1
#       define DDS_MULTIPLY_BY_2        0x2
#       define DDS_MULTIPLY_BY_3        0x3
#       define DDS_MULTIPLY_BY_4        0x4
#       define DDS_MULTIPLY_BY_6        0x6
#       define DDS_MULTIPLY_BY_8        0x8
#       define DDS_MULTIPLY_BY_10       0xA
#       define DDS_MULTIPLY_BY_16       0x10
#define COM_SET_DDS_PERIOD      0x49
#define COM_SET_DDS_TUNEWORD    0x4A
#define	COM_REQDTFWNUM		0x4F
#define COM_REQASSEMB		0xF4
#define	COM_REQHWFAB		0xF5
#define	COM_REQTFPMODEL		0xF6
#define	COM_REQSERNUM		0xFE

/*-------------------------------------------------------------*/
/* bc635PCI STATUS bits (from TIME0, EVENT0 registers) */

#define STAT_FLYWHEELING	0x01
#define STAT_TIME_OFFSET	0x02
#define STAT_FREQ_OFFSET	0x04

/*-------------------------------------------------------------*/
/* bc635PCI CONRTOL register bits */

#define CTRL_LOCKEN		0x01
#define CTRL_EVSOURCE		0x02
#define CTRL_EVSENSE		0x04
#define CTRL_EVENTEN		0x08
#define CTRL_STREN		0x10
#define CTRL_STRMODE		0x20
#define CTRL_F10		0x00
#define CTRL_F5			0x40
#define CTRL_F1			0x80
#define CTRL_FMASK		0xC0

/*-------------------------------------------------------------*/
/* bc635PCI ACK register bits */

#define ACK_COMMAND		0x01
#define ACK_PACKET		0x04
#define ACK_ACTION		0x80

/*-------------------------------------------------------------*/
/* bc635PCI INTSTAT and MASK register bits */
		
#define INT_EVENT		0x01
#define INT_PERIODIC		0x02
#define INT_STROBE		0x04
#define INT_1PPS		0x08
#define INT_PACKET		0x10
#define INT_EVENT2		0x20
#define INT_EVENT3		0x40
#define INT_BITMASK		0x7F

//---------------------------------------------------------------
// bc635/637PCI V2 hardware revision ID

#define BC_PCI_V2_REV_ID_START  0x20
#define BC_PCI_V2_REV_ID_END    0x2F
